#include "includes.h"


#define GEAR_GREEN_LEFT    0
#define GEAR_GREEN_NOSE    1
#define GEAR_GREEN_RIGHT   2
#define GEAR_MOVEMENT      3

#define FLAPS_20_PERCENT   4
#define FLAPS_40_PERCENT   5
#define FLAPS_60_PERCENT   6
#define FLAPS_80_PERCENT   7
#define FLAPS_100_PERCENT  8
#define FLAPS_MOVEMENT     9


typedef struct
{
 int image_ndx;
 LPC_GPIO_TypeDef   *port;
 unsigned long      mask;
} LED_STRUCT;


const LED_STRUCT leds[] = {

  { 3,  LPC_GPIO3,(0x1<<0)  /*9*/ },
  { 1,  LPC_GPIO1,(0x1<<0)  /*7*/ },
  { 1,  LPC_GPIO1,(0x1<<1)  /*6*/ },
  { 1,  LPC_GPIO1,(0x1<<2)  /*8*/ },
  { 0,  LPC_GPIO0,(0x1<<11) /*4*/ },
  { 2,  LPC_GPIO2,(0x1<<10) /*3*/ },
  { 0,  LPC_GPIO0,(0x1<<8)  /*2*/ },
  { 2,  LPC_GPIO2,(0x1<<9)  /*1*/ },
  { 2,  LPC_GPIO2,(0x1<<2)  /*0*/ },
  { 1,  LPC_GPIO1,(0x1<<10) /*5*/ },
  { 1,  LPC_GPIO1,(0x1<<4)  /*19*/ },
  { 1,  LPC_GPIO1,(0x1<<11) /*18*/ },
  { 3,  LPC_GPIO3,(0x1<<2)  /*17*/ },
  { 1,  LPC_GPIO1,(0x1<<5)  /*16*/ },
  { 1,  LPC_GPIO1,(0x1<<6)  /*15*/ },
  { 1,  LPC_GPIO1,(0x1<<7)  /*14*/ },
  { 3,  LPC_GPIO3,(0x1<<3)  /*13*/ },
  { 2,  LPC_GPIO2,(0x1<<6)  /*12*/ },
  { 2,  LPC_GPIO2,(0x1<<0)  /*11*/ },
  { 3,  LPC_GPIO3,(0x1<<1) /*10*/ }

};

void leds_init(void)
{
  LPC_GPIO2->DIR |= (0x1<<3);
  LPC_GPIO2->DATA &= ~(0x1<<3);

int i;
    for(i=0;i<20;i++)
    {
      leds[i].port->DIR |= leds[i].mask;
      leds[i].port->DATA |= leds[i].mask;
    }

}

void led_control(int led, int state)
{

   if(led>=0 && led <20)
   {
      if(state)
        leds[led].port->DATA &= ~leds[led].mask;
      else
        leds[led].port->DATA |= leds[led].mask;
   }
}


/************ Indication buffer ***************/
#define DEBUG_MODE 0

#define STX           '{'
#define ETX           '}'
#define RBUF_LEN       160
char rbuf[RBUF_LEN];
int  rcnt;

short gearl=0,gearn=0,gearr=0,flaps=0;


char seven_segments_leds[16][8] = {
#if 1
  /* indicators lines from simulator
  (it is thinking that sends to the 7 segments indicator lines)
  */
  {' ',' ',' ',' ',' ',' ',' ',' '}, /* 00                        15 */
  {'V','E','R','T','S',' ',' ',' '}, /* 01 VS[5]                  14 */
  {'A','L','T','I','T','I','A','S'}, /* 02 ALT(5)      IAS(3)     13 */
  {'F','F','F','F','H','H','H','H'}, /* 03 PRESS(4)    HPA(4)     12 */
  {'D','D','D','D','E','E','E','E'}, /* 04 DME1(4)     DME2(4)    11 */
  {'C','C','C','C','X','X','X','X'}, /* 05 DME_DIST(4) XPND(4)    10 */
  {'A','A','A','A','A','M','M','M'}, /* 06 DME_FREQ(5) MM_HG(3)   09 */
  {'9','9','9','9','B','B','B','B'}, /* 07 ADF1(4)     ADF2(4)    08 */
  {'8','8','8','8','8','H','D','G'}, /* 08 NAV2 STB(5) HDG-AP(3)  07 */
  {'7','7','7','7','7','C','R','2'}, /* 09 NAV2(5)     CRS2  (3)  06 */
  {'6','6','6','6','6','C','R','S'}, /* 10 NAV1_STB(5) CRS-AP(3)  05 */
  {'5','5','5','5','5','C','R','1'}, /* 11 NAV1(5)     CRS1  (3)  04 */
  {'4','4','4','4','4','G','G','R'}, /* 12 COM2_STB(5) GEARR (3)  03 */
  {'3','3','3','3','3','G','G','C'}, /* 13 COM2(5)     GEARC (3)  02 */
  {'2','2','2','2','2','G','G','L'}, /* 14 COM1_STB(5) GEARL (3)  01 */
  {'1','1','1','1','1','F','L','P'}  /* 15 COM1(5)     FLAPS (3)  00 */
#else
  {' ',' ',' ',' ',' ',' ',' ',' '}, /* 00                        15 */
  {'0','0','0','0','0',' ',' ',' '}, /* 01 VS[5]                  14 */
  {'0','0','0','0','0','0','0','0'}, /* 02 ALT(5)      IAS(3)     13 */
  {'0','0','0','0','0','0','0','0'}, /* 03 PRESS(4)    HPA(4)     12 */
  {'0','0','0','0','0','0','0','0'}, /* 04 DME1(4)     DME2(4)    11 */
  {'0','0','0','0','0','0','0','0'}, /* 05 DME_DIST(4) XPND(4)    10 */
  {'0','0','0','0','0','0','0','0'}, /* 06 DME_FREQ(5) MM_HG(3)   09 */
  {'0','0','0','0','0','0','0','0'}, /* 07 ADF1(4)     ADF2(4)    08 */
  {'0','0','0','0','0','0','0','0'}, /* 08 NAV2 STB(5) HDG-AP(3)  07 */
  {'0','0','0','0','0','0','0','0'}, /* 09 NAV2(5)     CRS2  (3)  06 */
  {'0','0','0','0','0','0','0','0'}, /* 10 NAV1_STB(5) CRS-AP(3)  05 */
  {'0','0','0','0','0','0','0','0'}, /* 11 NAV1(5)     CRS1  (3)  04 */
  {'0','0','0','0','0','0','0','0'}, /* 12 COM2_STB(5) GEARR (3)  03 */
  {'0','0','0','0','0','0','0','0'}, /* 13 COM2(5)     GEARC (3)  02 */
  {'0','0','0','0','0','0','0','0'}, /* 14 COM1_STB(5) GEARL (3)  01 */
  {'0','0','0','0','0','0','0','0'}  /* 15 COM1(5)     FLAPS (3)  00 */
#endif
};

  /* leds state from simulator (all of 64)*/
char single_leds[64] =
{
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0',
    '0','0','0','0','0','0','0','0'
};



void parser(char ch)
{
       unsigned short tmpu16;
       switch(ch)
        {
            case '@' :
              system_info();
            break;

            case STX:
                rcnt = 0;
            break;

            case ETX:
                rbuf[rcnt] = 0x00;

                if(rbuf[0] == '7')
                    memcpy(seven_segments_leds,rbuf+3,128);
                if(rbuf[0] == 'L')
                    memcpy(single_leds,rbuf+3,64);

                if(rbuf[0] == 'D')
                {
                   device_detect_response();
                }
                else
                {
                   tmpu16 = '!';
                   xQueueSend(disp_queue,&tmpu16,0);
                }
            break;

            default:
                if(rcnt<RBUF_LEN)
                    rbuf[rcnt++] = ch;
        }

}

char *radio_line1_modification(char *line, char mode, char tune_step, char stb_act)
{
  switch(mode)
  {

    case MODE_COM1:
       strcpy(line,"COM1    000.000 ");
       memcpy(&line[8],&seven_segments_leds[15][0],3);
       memcpy(&line[12],&seven_segments_leds[15][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';
    break;

    case MODE_COM2:
       strcpy(line,"COM2    000.000 ");
       memcpy(&line[8],&seven_segments_leds[13][0],3);
       memcpy(&line[12],&seven_segments_leds[13][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';
    break;

    case MODE_NAV1:
       strcpy(line,"NAV1    000.000 ");
       memcpy(&line[8],&seven_segments_leds[11][0],3);
       memcpy(&line[12],&seven_segments_leds[11][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';
    break;

    case MODE_NAV2:
       strcpy(line,"NAV2    000.000 ");
       memcpy(&line[8],&seven_segments_leds[9][0],3);
       memcpy(&line[12],&seven_segments_leds[9][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';
    break;

    case MODE_ADF:
          if(stb_act)
          {
              strcpy(line,"ADF2      000   ");
              memcpy(&line[9],&seven_segments_leds[7][4],4);
          }
          else
          {
              strcpy(line,"ADF1      000   ");
              memcpy(&line[9],&seven_segments_leds[7][0],4);
          }
    break;

    case MODE_DME:
          if(stb_act)
          {
              strcpy(line,"DME1    000.0 nm");
              memcpy(&line[8],&seven_segments_leds[4][0],3);
              line[12] = seven_segments_leds[4][3];

          }
          else
          {
              strcpy(line,"DME     000.000 ");
              memcpy(&line[8],&seven_segments_leds[6][0],3);
              memcpy(&line[12],&seven_segments_leds[6][3],2);
              if(line[13] == '2' || line[13] == '7')
                  line[14] = '5';
          }

    break;

    case MODE_TXPND:
             strcpy(line,"XPDR     0000   ");
             memcpy(&line[9], &seven_segments_leds[5][4],4);
    break;

    case MODE_NAV_CRS:
          if(stb_act)
          {
            strcpy(line,"NAV2 CRS   000  ");
            memcpy(&line[11], &seven_segments_leds[9][5],3);
          }
          else
          {
            strcpy(line,"NAV1 CRS   000  ");
            memcpy(&line[11], &seven_segments_leds[11][5],3);
          }
          if(tune_step)
          {
             line[10] = '<';
             line[14] = '>';
          }
    break;
  }
  return line;
}

char *radio_line2_modification(char *line, char mode, char tune_step, char stb_act)
{
  switch(mode)
  {

    case MODE_COM1:
       if(tune_step)
         strcpy(line,"STBY   <000>000 ");
       else
         strcpy(line,"STBY    000<000>");

       memcpy(&line[8],&seven_segments_leds[14][0],3);
       memcpy(&line[12],&seven_segments_leds[14][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';
    break;


    case MODE_COM2:
       if(tune_step)
         strcpy(line,"STBY   <000>000 ");
       else
         strcpy(line,"STBY    000<000>");
       memcpy(&line[8],&seven_segments_leds[12][0],3);
       memcpy(&line[12],&seven_segments_leds[12][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';

    break;

    case MODE_NAV1:
       if(tune_step)
         strcpy(line,"STBY   <000>000 ");
       else
         strcpy(line,"STBY    000<000>");

       memcpy(&line[8],&seven_segments_leds[10][0],3);
       memcpy(&line[12],&seven_segments_leds[10][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';


    break;


    case MODE_NAV2:
       if(tune_step)
         strcpy(line,"STBY   <000>000 ");
       else
         strcpy(line,"STBY    000<000>");
       memcpy(&line[8],&seven_segments_leds[8][0],3);
       memcpy(&line[12],&seven_segments_leds[8][3],2);
       if(line[13] == '2' || line[13] == '7')
           line[14] = '5';

    break;


    case MODE_ADF:
         strcpy(line,"                ");
         line[10 + tune_step] = '^';
    break;

    case MODE_DME:
          if(stb_act)
          {
              strcpy(line,"DME2    000.0 nm");
              memcpy(&line[8],&seven_segments_leds[4][4],3);
              line[12] = seven_segments_leds[4][7];
          }
          else
          {
              strcpy(line,"000.0 nm        ");
              memcpy(&line[0],&seven_segments_leds[5][0],3);
              line[4] = seven_segments_leds[5][3];

              if(tune_step)
                  line[10] = '^';
              else
                  line[14] = '^';
          }
    break;

    case MODE_TXPND:
         strcpy(line,"                ");
         line[9 + tune_step] = '^';
    break;

    case MODE_NAV_CRS:
       if(!tune_step)
         strcpy(line,"<00.00 0000 000>");
       else
         strcpy(line," 00.00 0000 000 ");

       memcpy(&line[1],&seven_segments_leds[3][0],2);
       memcpy(&line[4],&seven_segments_leds[3][2],2);

       memcpy(&line[7],&seven_segments_leds[3][4],4);
       memcpy(&line[12],&seven_segments_leds[6][5],3);


    break;
  }
  return line;
}


char *ap_line1_modification(char *line, char mode)
{


  switch(mode)
  {
    case AP_IAS:
                // 0123456701234567
      strcpy(line," 000 [000]  000 ");
    break;

    case AP_HDG:
                // 0123456701234567
      strcpy(line," 000  000  [000]");
    break;

    case AP_CRS:
                // 0123456701234567
      strcpy(line,"[000] 000   000 ");
    break;


    default:
                // 0123456701234567
      strcpy(line," 000  000   000 ");

  }

  memcpy(&line[1],&seven_segments_leds[10][5],3); /*CRS*/
  memcpy(&line[6],&seven_segments_leds[2][5],3); /*IAS*/
  memcpy(&line[12],&seven_segments_leds[8][5],3); /*HDG*/

  return line;
}


char *ap_line2_modification(char *line, char mode)
{


  switch(mode)
  {
    case AP_ALT:
                // 0123456701234567
      strcpy(line,"  0000   [00000]");
    break;


    case AP_VS:
                // 0123456701234567
      strcpy(line,"[ 0000]   00000 ");
    break;

    default:
                // 0123456701234567
      strcpy(line,"  0000    00000 ");

  }

  memcpy(&line[1],&seven_segments_leds[1][0],5);  /*VS*/
  memcpy(&line[10],&seven_segments_leds[2][0],5); /*ALT*/

  if(line[1] == ' ' || line[1] == '0')
        line[1] = '+';

  return line;
}

void gear_n_flaps(void)
{
  char line[6];
  memcpy(line,&seven_segments_leds[14][5],3);
  gearl = atoi(line);

  memcpy(line,&seven_segments_leds[13][5],3);
  gearn = atoi(line);

  memcpy(line,&seven_segments_leds[12][5],3);
  gearr = atoi(line);

  memcpy(line,&seven_segments_leds[15][5],3);
  flaps = atoi(line);

    led_control(FLAPS_20_PERCENT, flaps >= 200);
    led_control(FLAPS_40_PERCENT, flaps >= 400);
    led_control(FLAPS_60_PERCENT, flaps >= 600);
    led_control(FLAPS_80_PERCENT, flaps >= 800);
    led_control(FLAPS_100_PERCENT, flaps >= 999);

    led_control(GEAR_GREEN_LEFT, gearl >= 999);
    led_control(GEAR_GREEN_NOSE, gearn >= 999);
    led_control(GEAR_GREEN_RIGHT, gearr >= 999);
}

void check_movements(void)
{
  static char tim = 0;
  static short prev_pos_gearl = 0,
               prev_pos_gearn = 0,
               prev_pos_gearr = 0,
               prev_pos_flaps = 0;
  if(++tim >= 20)
  {
    tim = 0;

    if(prev_pos_gearl == gearl)
    {
      led_control(GEAR_MOVEMENT, 0);
    }
    else
    {
      led_control(GEAR_MOVEMENT, 1);
    }

    if(prev_pos_gearn == gearn)
    {
      led_control(GEAR_MOVEMENT, 0);
    }
    else
    {
      led_control(GEAR_MOVEMENT, 1);
    }



    if(prev_pos_gearr == gearr)
    {
      led_control(GEAR_MOVEMENT, 0);
    }
    else
    {
      led_control(GEAR_MOVEMENT, 1);
    }


    if(prev_pos_flaps == flaps)
    {
      led_control(FLAPS_MOVEMENT, 0);
    }
    else
    {
      led_control(FLAPS_MOVEMENT, 1);
    }

    prev_pos_gearl = gearl;
    prev_pos_gearn = gearn;
    prev_pos_gearr = gearr;
    prev_pos_flaps = flaps;
  }

}


void refresh_leds(void)
{
   int i;
   //for(i=0;i<3;i++)
   //  led_control(i,(single_leds[i] & 1));

   for(i=10;i<20;i++)
     led_control(i,(single_leds[i] & 1));

}



void disp_task( void *params )
{
    char line[17];
    unsigned short ev;
    int result;
    leds_init();

    INIT_LCD_PINS();
    lcd_init_1();
    lcd_init_2();

	lcd_puts(0,0, " -=FlyingCat=-  ");
	lcd_puts(0,1, "RadioStack Panel");

	lcd_puts(1,0, " -=FlyingCat=-  ");
	lcd_puts(1,1, "AutoPilot  Panel");
    lcd_backlight_on();

    vTaskDelay(1000);

    while(1)
    {
       result = xQueueReceive(disp_queue,&ev,1000);

	    #if 1
         lcd_puts(0,0, radio_line1_modification(line,
                                                radio_mode,
                                                radio_step,
                                                radio_stb_act));
        #endif
        #if 1
	     lcd_puts(0,1, radio_line2_modification(line,
                                                radio_mode,
                                                radio_step,
                                                radio_stb_act));
        #endif

        #if 1
	     lcd_puts(1,0, ap_line1_modification(line,
                                                ap_mode));
        #endif

        #if 1
	     lcd_puts(1,1, ap_line2_modification(line,
                                                ap_mode));
        #endif

        #if 1
         gear_n_flaps();
        #endif


        #if 1
         refresh_leds();
        #endif

        if(result)
        {
          ev &= 0xFF;
          xQueueSend(com_queue,&ev,100);
        }

    }
}


/*
    0       1       2       3       4       5       6       7       8        9      10       11     12      13      14      15
    01234567012345670123456701234567012345670123456701234567012345670123456701234567012345670123456701234567012345670123456701234567
{7SG00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000099900000999000009990000099912345500}
              1         2         3         4         5         6
    0123456789012345678901234567890123456789012345678901234567890123
{LED0000000000000000001000000000000000000000000000000000000000000000}


*/