#include "includes.h"

volatile Int32U Ticks;
Int32U CriticalSecCntr;
xQueueHandle com_queue = NULL;
xQueueHandle disp_queue = NULL;
char comm_buffer[USB_BUFFER_SIZE];

static void prvSetupHardware( void );
void vUSBTask( void *pvParameters );
void prvUSBTask( void *pvParameters );



int main( void )
{
	/* Configure the hardware for use by this demo. */
	prvSetupHardware();

        /* Start up task creating. */
	xTaskCreate( start_up_task,
                    ( signed char * ) "StartUp",
                    2*configMINIMAL_STACK_SIZE,
                    ( void * ) NULL, tskIDLE_PRIORITY+1,
                    NULL );
    /* Start the scheduler. */
	vTaskStartScheduler();
    /* Will only get here if there was insufficient memory to create the idle
       task.  The idle task is created within vTaskStartScheduler(). */
	for( ;; );
}

void start_up_task(void * pvParameters )
{
    pvParameters = pvParameters;

    xTaskCreate( prvUSBTask,
            (signed char * )   "USB",
            3*configMINIMAL_STACK_SIZE,
            ( void * ) NULL,
            tskIDLE_PRIORITY, NULL );

    com_queue = xQueueCreate(2,5);
    disp_queue = xQueueCreate(2,1);

    inputs64_init();

    xTaskCreate( disp_task,
            (signed char * ) "DISP",
            2*configMINIMAL_STACK_SIZE,
            ( void * ) NULL,
            tskIDLE_PRIORITY, NULL );

    vTaskDelay(2000);

    discrete_task(NULL);
}

void prvUSBTask( void *pvParameters )
{
short ev,i;
Boolean CdcConfigureStateHold;
int len_to_send;
int len_recv;


__disable_interrupt();
  UsbCdcInit();
 __enable_interrupt();
  // Soft connection enable
  USB_ConnectRes(TRUE);

  CdcConfigureStateHold = !IsUsbCdcConfigure();
  int last_check_time = xTaskGetTickCount();
  while(1)
  {
    if (IsUsbCdcConfigure())
    {
      len_recv = UsbCdcRead((unsigned char*)comm_buffer,USB_BUFFER_SIZE);
      if(len_recv)
      {
        for(i=0;i<len_recv;i++)
                    parser(comm_buffer[i]);
      }
      if(xQueueReceive(com_queue,&ev,0))
      {
           ev &= 0xffff;
           if(ev & 0x0F00)
              len_to_send = sprintf(comm_buffer,"{%03X}", ev & 0xFFF);
           else
              {
                  len_to_send = 1;
                  comm_buffer[0] = ev & 0xFF;
              }

      }  else
          len_to_send = 0;

      if(len_to_send)
      {
        last_check_time = xTaskGetTickCount();

         if(len_to_send)
         {
           UsbCdcWrite((unsigned char*)comm_buffer,len_to_send);
         }
      }
    }
    else
    {
      if(CdcConfigureStateHold == TRUE)
      {
        CdcConfigureStateHold = FALSE;
      }
    }
  }



}



/*************************************************************************
 * Function Name: NVIC_IntEnable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Enable interrup at NVIC
 *************************************************************************/
void NVIC_IntEnable(Int32U IntNumber)
{
volatile unsigned long * pNVIC_SetEn = &SETENA0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_SetEn += IntNumber/32;
  *pNVIC_SetEn = (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_IntDisable
 * Parameters: IntNumber - Interrup number
 * Return: void
 * Description: Disables interrup at NVIC
 *************************************************************************/
void NVIC_IntDisable(Int32U IntNumber)
{
volatile unsigned long * pNVIC_ClrEn = &CLRENA0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_ClrEn += IntNumber/32;
  *pNVIC_ClrEn = (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number
 * Return: void
 * Description:Clear pending interrupt at NVIC
 *************************************************************************/
void NVIC_ClrPend(Int32U IntNumber)
{
volatile unsigned long * pNVIC_ClrPend = &CLRPEND0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_ClrPend += IntNumber/32;
  *pNVIC_ClrPend = (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number, Interrupt Priority
 * Return: void
 * Description:Sets Interrupt priority
 *************************************************************************/
void NVIC_IntPri(Int32U IntNumber, Int8U Priority)
{
volatile Int8U * pNVIC_IntPri = (Int8U *)&IP0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;
  pNVIC_IntPri += IntNumber;
  *pNVIC_IntPri = Priority;
}


void InitClock(Int32U clock, Int32U ahbdiv)
{
  /*Sys Oscilator Enable*/
  //SYSOSCCLTRL = (MAIN_OSC_FREQ>(20MHZ))?(0x2):(0x0);
    SYSOSCCTRL = (MAIN_OSC_FREQ>(20MHZ))?(0x2):(0x0);
  /*Power Up SYS Oscilator*/
  PDRUNCFG_bit.SYSOSC_PD = 0;
  /*Enable Internal RC oscilator*/
  PDRUNCFG_bit.IRC_PD = 0;
  /*Select internal RC oscilator for
    Sys clock source*/
  MAINCLKUEN = 0;
  MAINCLKSEL = 0;
  MAINCLKUEN = 1;
  /*Configure SYS PLL*/
  /*Power Down SYS PLL*/
  PDRUNCFG_bit.SYSPLL_PD = 1;
  /*Select Sys Oscilator for
    SYS PLL source*/
  SYSPLLCLKUEN = 0;
  SYSPLLCLKSEL = 1;
  SYSPLLCLKUEN = 1;
  /*Calc M*/
  Int32U m = clock/MAIN_OSC_FREQ - 1;

  assert(m<32);
  /*Configure PLL frequency*/
  SYSPLLCTRL =  (m)    /*MSEL*/
             |  (0<<5) /*PSEL = 1*/
             |  (0<<7) /*DIRECT = 0*/
             |  (0<<8); /*BYPASS=0*/

  /*Power Up PLL*/
  PDRUNCFG_bit.SYSPLL_PD = 0;
  /*Set Sys AHB Clock devider*/
  SYSAHBCLKDIV_bit.DIV = ahbdiv;
  /*Wain until PLL locks*/
  while(!(SYSPLLSTAT_bit.LOCK));
  /*Select Sys PLL Output for
    Sys clock source*/
  MAINCLKUEN = 0;
  MAINCLKSEL = 3;
  MAINCLKUEN = 1;
}


void GpioInit(void)
{
  // Set to inputs
  GPIO0DIR = \
  GPIO1DIR = \
  GPIO2DIR = \
  GPIO3DIR = 0;

  // Reset all GPIO pins to default primary function
  IOCON_PIO2_6 = \
  IOCON_PIO2_0 = \
  IOCON_RESET_PIO0_0 = \
  IOCON_PIO0_1 = \
  IOCON_PIO1_8 = \
  IOCON_PIO0_2 = \
  IOCON_PIO2_7 = \
  IOCON_PIO2_8 = \
  IOCON_PIO2_1 = \
  IOCON_PIO0_3 = \
  IOCON_PIO2_4 = \
  IOCON_PIO2_5 = \
  IOCON_PIO0_6 = \
  IOCON_PIO2_9 = \
  IOCON_PIO2_10 = \
  IOCON_PIO2_2 = \
  IOCON_PIO0_8 = \
  IOCON_SWCLK_PIO0_10 = \
  IOCON_PIO2_11 = \
  IOCON_PIO3_0 = \
  IOCON_PIO2_3 = \
  IOCON_PIO1_5 = \
  IOCON_PIO1_6 = \
  IOCON_PIO1_9 = \
  IOCON_PIO3_1 = \
  IOCON_PIO3_2 = \
  IOCON_PIO3_3 = 0x50;

  IOCON_PIO1_10 = \
  IOCON_SWDIO_PIO1_3 = \
  IOCON_PIO1_4 = \
  IOCON_PIO1_11 = 0xD0;

  IOCON_R_PIO1_0 = 0x81;
  IOCON_R_PIO1_1 = 0x81;
  IOCON_R_PIO1_2 = 0x81;
  IOCON_R_PIO0_11 = 0x81;
  IOCON_PIO1_10 = 0x80;

  IOCON_PIO0_9 = \
  IOCON_PIO0_4 = \
  IOCON_PIO0_5 = \
  IOCON_PIO0_7 = 0x00;

  IOCON_SCKLOC = 0x00;
}


Int32U SYS_GetMainClk(void)
{
Int32U Clk;
  switch(MAINCLKSEL_bit.SEL)
  {
    case 0:
       Clk = I_RC_OSC_FREQ;
    break;
    case 1:
      Clk = MAIN_OSC_FREQ;
    break;
    case 2:
      Clk = WDT_OSC_FREQ;
    break;
    case 3:
      switch(SYSPLLCLKSEL_bit.SEL)
      {
        case 0:
          Clk = I_RC_OSC_FREQ;
        break;
        case 1:
          Clk = MAIN_OSC_FREQ;
        break;
        case 2:
          Clk = WDT_OSC_FREQ;
        break;

        default:
          Clk = 0;
        break;
      }


  if(!SYSOSCCTRL_bit.BYPASS)
        Clk *= (SYSPLLCTRL_bit.MSEL+1);

    break;

    default:
      Clk = 0;
    break;
  }
  return Clk;
}

/**************************************************************
    ,   
                        system tick
 **************************************************************/

void CT32B0_Init(Int32U tps)
{
  Ticks = 0;
  // Enable TIM0 clocks
  SYSAHBCLKCTRL_bit.CT32B0 = 1;

  // Init Time0
  TMR32B0TCR_bit.CE = 0;     // counting  disable
  TMR32B0TCR_bit.CR = 1;     // set reset
  TMR32B0TCR_bit.CR = 0;     // release reset
  TMR32B0CTCR_bit.CTM = 0;   // Timer Mode: every rising PCLK edge
  TMR32B0MCR_bit.MR0I = 1;   // Enable Interrupt on MR0
  TMR32B0MCR_bit.MR0R = 1;   // Enable reset on MR0
  TMR32B0MCR_bit.MR0S = 0;   // Disable stop on MR0
  // set timer 0 period
  TMR32B0PR = 0;
  TMR32B0MR0 = (SYS_GetMainClk()/(SYSAHBCLKDIV))/(tps);
  //  timer 0 interrupt
  TMR32B0IR_bit.MR0INT = 1;  // clear pending interrupt
  TMR32B0TCR_bit.CE = 1;     // counting Enable
   /*Enable NVIC TMR0 Interrupt*/
  NVIC_IntEnable(NVIC_CT32B0);
  NVIC_IntPri(NVIC_CT32B0,16);
}



void system_info(void)
{
    char *p,*buf = pvPortMalloc(300);
    int len;
    if(configUSE_TRACE_FACILITY && buf)
    {
      len = sprintf(buf,"\r\nHeap memory total/free .............. %ld/%ld\r\n",
      configTOTAL_HEAP_SIZE, xPortGetFreeHeapSize());
      UsbCdcWrite((unsigned char*)buf,len);
      vTaskList( (signed char *) buf );

              p = buf;
              while(*p)
              {
               UsbCdcWrite((unsigned char*)p++,1);
              }

    }
      vPortFree( buf );

}

void device_detect_response(void)
{
    const char det_response[] = "!{F"__DEVICE_ID__"}";
    UsbCdcWrite((unsigned char*)det_response,6);
}



void prvSetupHardware( void )
{

/*Init clock*/
  InitClock(132MHZ, 2);
  /*Enable GPIO Clock*/
  SYSAHBCLKCTRL_bit.GPIO = 1;
  /*Enable IOCON Clock*/
  SYSAHBCLKCTRL_bit.IOCON = 1;
  /*Set all pins as input ports*/

  //init_timer16(0, TIME_INTERVAL);
  GpioInit();
    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);
   // LPC_GPIO3->DIR |= ((1<<2) | (1<<1));

}
/*-----------------------------------------------------------*/

/*************************************************
   -   
**************************************************/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will get called if a task overflows its stack. */

	( void ) pxTask;
	( void ) pcTaskName;

	for( ;; );
}

/************************************************************
        
              
 ************************************************************/
void vConfigureTimerForRunTimeStats( void )
{

const unsigned long TCR_COUNT_RESET = 2, CTCR_CTM_TIMER = 0x00, TCR_COUNT_ENABLE = 0x01;

	/* This function configures a timer that is used as the time base when
	collecting run time statistical information - basically the percentage
	of CPU time that each task is utilising.  It is called automatically when
	the scheduler is started (assuming configGENERATE_RUN_TIME_STATS is set
	to 1). */

    SYSAHBCLKCTRL_bit.CT32B0 = 1;

	/* Reset Timer 0 */
	TIM0->TCR = TCR_COUNT_RESET;

	/* Just count up. */
	TIM0->CTCR = CTCR_CTM_TIMER;

	/* Prescale to a frequency that is good enough to get a decent resolution,
	but not too fast so as to overflow all the time. */
	TIM0->PR =  ( configCPU_CLOCK_HZ / 10000UL ) - 1UL;

	/* Start the counter. */
	TIM0->TCR = TCR_COUNT_ENABLE;


}

/************************************************************
        
                   CRP2
 ************************************************************/
#if RELEASE_BUILDING
   #pragma location="CodeReadProtection"
   __root const unsigned long crp_data = 0x87654321; // Code read protection
#else
   #pragma location="CodeReadProtection"
   __root const unsigned long crp_data = 0xFFFFFFFF;
#endif



size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{

  return 0;
}




